@@ -24,7 +24,8 @@ from collections.abc import MutableMapping, Mapping
 import email
 import email.policy
 from http.client import (HTTPS_PORT, HTTP_PORT, NO_CONTENT, NOT_MODIFIED)
-from select import select, EPOLLIN, EPOLLOUT
+from select import EPOLLIN, EPOLLOUT
+import select
 
 try:
     import asyncio
@@ -32,7 +33,7 @@ except ImportError:
     asyncio = None
 
     
-__version__ = '2.0'
+__version__ = '2.1'
 
 log = logging.getLogger(__name__)
 
@@ -92,25 +93,27 @@ class PollNeeded(tuple):
     def poll(self, timeout=None):
         '''Wait until fd is ready for requested IO
 
-        This is a convenince function that uses `~select.select` to wait until
+        This is a convenince function that uses `~select.poll` to wait until
         `.fd` is ready for requested type of IO.
 
         If *timeout* is specified, return `False` if the timeout is exceeded
         without the file descriptor becoming ready.
         '''
 
-        read_fds = (self.fd,) if self.mask & EPOLLIN else ()
-        write_fds = (self.fd,) if self.mask & EPOLLOUT else ()
-
-        log.debug('calling select with %s, %s', read_fds, write_fds)
-        if timeout is None:
-            (read_fds, write_fds, _) =  select(read_fds, write_fds, ())
+        poll = select.poll()
+        if self.mask & EPOLLIN:
+            poll.register(self.fd, select.POLLIN)
+        if self.mask & EPOLLOUT:
+            poll.register(self.fd, select.POLLOUT)
+            
+        log.debug('calling poll')
+        if timeout:
+            return bool(poll.poll(timeout*1000)) # convert to ms
         else:
-            (read_fds, write_fds, _) =  select(read_fds, write_fds, (), timeout)
+            return bool(poll.poll())
+            
 
-        return bool(read_fds) or bool(write_fds)
 
-    
 class HTTPResponse:
     '''
     This class encapsulates information about HTTP response.  Instances of this
@@ -552,13 +555,17 @@ class HTTPConnection:
 
         fd = self._sock.fileno()
         while True:
-            if not select((), (fd,), (), 0)[1]:
+            try:
+                len_ = self._sock.send(buf)
+                # An SSL socket has the nasty habit of returning zero
+                # instead of raising an exception when in non-blocking
+                # mode.
+                if len_ == 0:
+                    raise BlockingIOError()
+            except (socket.timeout, ssl.SSLWantWriteError, BlockingIOError):
                 log.debug('yielding')
                 yield PollNeeded(fd, EPOLLOUT)
                 continue
-
-            try:
-                len_ = self._sock.send(buf)
             except BrokenPipeError:
                 raise ConnectionClosed('found closed when trying to write')
             except OSError as exc:
@@ -568,9 +575,11 @@ class HTTPConnection:
                 else:
                     raise
             except InterruptedError:
+                log.debug('interrupted')
                 # According to send(2), this means that no data has been sent
                 # at all before the interruption, so we just try again.
-                pass
+                continue
+            
             log.debug('sent %d bytes', len_)
             buf = buf[len_:]
             if len(buf) == 0:
@@ -631,9 +640,6 @@ class HTTPConnection:
         Return a `HTTPResponse` instance containing information about response
         status, reason, and headers. The response body data must be retrieved
         separately (e.g. using `.read` or `.readall`).
-
-        Even for a response with empty body, one of the body reading method must
-        be called once before the next response can be processed.
         '''
 
         log.debug('start')
@@ -715,9 +721,8 @@ class HTTPConnection:
             100 <= status < 200 or method == 'HEAD'):
             log.debug('no content by RFC')
             body_length = 0
-            self._in_remaining = 0
-            # for these cases, there isn't even a zero chunk we could read
-            self._encoding = IDENTITY_ENCODING
+            self._in_remaining = None
+            self._pending_requests.popleft()
 
         # Chunked doesn't require content-length
         elif self._encoding is CHUNKED_ENCODING:
@@ -732,10 +737,14 @@ class HTTPConnection:
             self._in_remaining = 0
 
         else:
-            self._in_remaining = int(header['Content-Length'])
-            body_length = self._in_remaining
+            body_length = int(header['Content-Length'])
+            if body_length:
+                self._in_remaining = body_length
+            else:
+                self._in_remaining = None
+                self._pending_requests.popleft()
 
-        log.debug('done (in_remaining=%d)', self._in_remaining)
+        log.debug('done (in_remaining=%s)', self._in_remaining)
 
         return HTTPResponse(method, path, status, reason, header, body_length)
     
